﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Text;
using System.Globalization;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Messages;
using Produmex.Foundation.Collections;
using Produmex.Foundation.Data.RestApiClient;
using Produmex.Foundation.Data.DbClient;
using System.Xml;
using System.Data.Common;
using Produmex.Sbo.Logex.Data.Utilities;
using System.Drawing;
//css_ref System.Drawing.dll;



// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_SalesDeliveryCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesDeliveryCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;

        // Output parameters
        // <none>

        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesDeliveryCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesDeliveryCreatedHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
        }

		#region WorkflowInstanceScriptBase Members

		static class Definitions
		{

			/// <summary>
			/// Interface DPD
			/// </summary>
			public const string DPD = "DPD";


			/// <summary>
			/// VIEW
			/// </summary>
			public const string VIEW_NAME= "PMX_HO_INTERFACES";
			public const string VIEW_COLUMN_INTERFACETYPE= "InterfaceType";
			public const string VIEW_COLUMN_DOCENTRY = "DocEntry";

			public const string VIEW_NAME_DPD = "PMX_HO_INTERFACES_DPD_DATA";

			/// <summary>
			/// WEBSERVICE
			/// </summary>
			public const string USERNAME = "hochikieurope";
			public const string PASSWORD = "hochiki108466";

			public const string URL_LOGIN = "https://api.dpd.co.uk/user/?action=login";
			public const string URL_SHIPMENT = "https://api.dpd.co.uk/shipping/shipment";
			public const string URL_LABEL = "https://api.dpd.co.uk/shipping/shipment/{0}/label/";



			public const string VIEW_PREFIX_COSIGNMENT = "CON_";			
			public const string VIEW_PREFIX_COLLECTION_DETAILS = "CD_";
			public const string VIEW_PREFIX_DELIVERY_DETAILS = "DD_";
			public const string VIEW_PREFIX_CONTACTDETAILS = "CD_";
			public const string VIEW_PREFIX_ADDRESS = "A_";
			public const string VIEW_PREFIX_NOTIFICATIONDETAILS= "ND_";



			public const string DPD_XMLNODE_ROOT = "root";
			public const string DPD_XMLNODE_COSIGNMENT = "consignment";
			public const string DPD_XMLNODE_COLLECTION_DETAILS = "collectionDetails";
			public const string DPD_XMLNODE_DELIVERY_DETAILS = "deliveryDetails";
			public const string DPD_XMLNODE_CONTACTDETAILS = "contactDetails";
			public const string DPD_XMLNODE_ADDRESS = "address";
			public const string DPD_XMLNODE_NOTIFICATIONDETAILS = "notificationDetails";
			public const string DPD_XMLNODE_DOCENTRY = "DocEntry";


			public const string DPD_CONVERT_TO_DECIMAL = "CON_totalWeight";
			public const string DPD_CONVERT_TO_DATE = "collectionDate";
			public const string DPD_CONVERT_TO_INT = "CON_numberOfParcels";
			public const string DPD_CONVERT_TO_BOOLEAN = "collectionOnDelivery,consolidate,CON_liability";

			public const string DPD_DATE_FORMAT = "yyyy-MM-dd";
			public const string DPD_DECIMAL_FORMAT = "0.00";


			public const string ODLN_PMX_PSSD = "U_PMX_PSSD";
			public const string ODLN_PMX_ = "0.00";


			public const string PAGESIZE_FOR_PRINTING = "A5";
			public const string PAGESIZE_FOR_PRINTING_STANDBY = "A4";



		}

		private List<string> ListColumnsDecimals;
		private List<string> ListColumnsDates;
		private List<string> ListColumnsInteger;
		private List<string> ListColumnsBoolean;

		private string m_printer = null;
		private string fileName = null;

		protected override void Execute()
        {
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

			string msgKey = null;
			ProdumexError pmxError = null;
			Message msg = null;


			string location = "";
			PmxOsePrinter printer = null;
			sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
				delegate (PmxDbConnection dbConn, object[] parameters)
				{
					PmxOsePrinterProvider prov = new PmxOsePrinterProvider(dbConn);
					printer = prov.GetPrinterForLocation(location, session.DeviceInstanceId, Definitions.PAGESIZE_FOR_PRINTING);

					if (printer == null && !String.IsNullOrEmpty(Definitions.PAGESIZE_FOR_PRINTING_STANDBY))
					{
						printer = prov.GetPrinterForLocation(location, session.DeviceInstanceId, Definitions.PAGESIZE_FOR_PRINTING_STANDBY);
					}

					return null;
				});
			m_printer = printer?.WindowsPrinterName;



			foreach (DocumentRef docRef in DocumentRefs.Get())
			{
				try
				{


					sboProviderService.InvokeMethodWithDbConnection<bool>(false, true, null, null, Export, session, docRef.DocEntry);
				}
				catch (ProdumexException ex)
				{
					s_log.Error(ex.ToString());
					msgKey = ex.Message;
					pmxError = ex.Error;
				}
				catch (Exception ex)
				{
					s_log.Error("", ex);
					msgKey = ex.Message;
				}
				if (msgKey != null)
				{
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", null,
							"MessageKey", msgKey,
							"PmxError", pmxError,
							"ShowButton", true));
					WaitForMessage();
				}
			}


		}

		/// <summary>
		/// Exports the specified data.
		/// </summary>

		/// <param name="conn">The conn.</param>
		/// <returns>
		/// [true] when export has to be done for this document
		/// </returns>
		public bool Export(PmxDbConnection conn, object[] parameters)
		{
			Session session = (Session)parameters[0];
			int docEntry = (int)parameters[1];


			//check if docEntry is in the view and get the interface type
			string query = BuildQuery.GetQueryGetDataFromView(docEntry, Definitions.VIEW_NAME, Definitions.VIEW_COLUMN_DOCENTRY, conn.DbTool);

			using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
			{
				while (!rs.EoF)
				{
					string interfaceType = null;
					interfaceType = Convert.ToString(rs[Definitions.VIEW_COLUMN_INTERFACETYPE]);
					switch (interfaceType)
					{
						case Definitions.DPD:
							return Interface_DPD(conn, session, docEntry);
						
						default:
							break;
					}
				}
			}
			return true;
		}


		/// <summary>
		/// Export Interface DHL.
		/// </summary>
		/// <param name="request">The request.</param>
		/// <param name="parameters">The parameters.</param>
		/// <param name="tcParameters">The tcParameters.</param>
		/// <param name="conn">The conn.</param>
		/// <returns></returns>

		public bool Interface_DPD(PmxDbConnection conn, Session session, int docEntry)
		{
			ListColumnsDecimals = Definitions.DPD_CONVERT_TO_DECIMAL.Split(',').ToList();
			ListColumnsDates = Definitions.DPD_CONVERT_TO_DATE.Split(',').ToList();
			ListColumnsInteger = Definitions.DPD_CONVERT_TO_INT.Split(',').ToList();
			ListColumnsBoolean = Definitions.DPD_CONVERT_TO_BOOLEAN.Split(',').ToList();
						
			//GeoSession?
			string geoSession = GetGeoSession(conn, session);
			if (string.IsNullOrEmpty(geoSession)) return false;

			
			//Shipment
			int? shipmentId = CreateShipment(conn, session, docEntry, geoSession);
			if (!shipmentId.HasValue) return false;

			string query = string.Format("UPDATE ODLN SET \"{0}\" = '{1}' WHERE \"{2}\" = {3} ", Definitions.ODLN_PMX_PSSD, shipmentId.ToString(), Definitions.DPD_XMLNODE_DOCENTRY, docEntry );
			SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);


			//Get label
			string printString= getLabel(conn, session, geoSession, shipmentId.Value);
			if (string.IsNullOrEmpty(printString)) return false;


			//Print label
			string outputDir = Path.GetTempPath();
			fileName = shipmentId.Value.ToString() + ".html";
			if (!outputDir.EndsWith(@"\"))
			{
				outputDir += @"\";
			}

			byte[] data = Encoding.ASCII.GetBytes(printString);
			System.IO.Directory.CreateDirectory(outputDir);
			fileName = Path.Combine(outputDir, fileName);

			System.IO.FileStream LabelFile = new System.IO.FileStream(fileName, System.IO.FileMode.Create);
			LabelFile.Write(data, 0, data.Length);
			LabelFile.Close();

			 
			return PrintDoc();

		}

		public string GetGeoSession(PmxDbConnection conn, Session session)
		{
			string geoSession = null;
			ServicePointManager.Expect100Continue = true;
			ServicePointManager.SecurityProtocol = (SecurityProtocolType)3072;
			ServicePointManager.DefaultConnectionLimit = 9999;

			RestApiBasicAuthorizationCredentials credentials = new RestApiBasicAuthorizationCredentials(Definitions.USERNAME, Definitions.PASSWORD);
			
			RestApiCient restApiClient = new RestApiCient(Definitions.URL_LOGIN);			
			RestApiResponse<WebServiceLoginResponse> response = restApiClient.DoAction<WebServiceRequest, WebServiceLoginResponse>(
				RestMethodType.Post,
				"",
				null,
				RestApiSerializationType.JSON, credentials);
			if (response != null)
			{
				if (string.IsNullOrEmpty(response.ErrorMessage) && response.ResponseHttpStatus != 0)
				{
					geoSession = response.ResponseObject.data.geoSession;
				}
				else
				{
					LogError(response.ErrorMessage);
					InformTheUserAboutError(response.ErrorMessage, session);
				}
			}
			else
			{
				LogError("The response wasn't a valid json string (Login).");
				InformTheUserAboutError("No label printing because of communication error (Login).", session);
			}

			return geoSession;
		}

		public int? CreateShipment(PmxDbConnection conn, Session session, int docEntry, string geoSession)
		{
			int? shipmentId = null;

			string json = CreateJsonContent(conn, docEntry);


			ServicePointManager.Expect100Continue = true;
			ServicePointManager.SecurityProtocol = (SecurityProtocolType)3072;
			ServicePointManager.DefaultConnectionLimit = 9999;

			byte[] postBytes = Encoding.UTF8.GetBytes(json);

			var httpWebRequest = (HttpWebRequest)WebRequest.Create(Definitions.URL_SHIPMENT);
			httpWebRequest.ContentType = "application/json";
			httpWebRequest.Method = "POST";
			httpWebRequest.Headers.Add("GeoSession", geoSession);


			Stream requestStream = httpWebRequest.GetRequestStream();
			requestStream.Write(postBytes, 0, postBytes.Length);
			requestStream.Close();

			RestApiResponse<WebServiceShipmentResponse> response = new RestApiResponse<WebServiceShipmentResponse>();
			var httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();

			using (var streamReader = new StreamReader(httpWebResponse.GetResponseStream()))
			{
				response.ResponseString = streamReader.ReadToEnd();

			}

			WebServiceShipmentResponse jsonResponse = RestJsonSerializer.JsonDeserialize<WebServiceShipmentResponse>(response.ResponseString);

			if (response != null)
			{
				if (jsonResponse.error== null && response.ResponseHttpStatus == 0)
				{
					shipmentId =  jsonResponse.data.shipmentId;
				}
				else
				{
					LogError(string.Format("obj:{0}, errormsg:{1}",jsonResponse.error[0].obj, jsonResponse.error[0].errorMessage));
					InformTheUserAboutError(string.Format("obj:{0}, errormsg:{1}", jsonResponse.error[0].obj, jsonResponse.error[0].errorMessage), session);
				}
			}	
			
			else
			{
				LogError("The response wasn't a valid json string (Shipment).");
				InformTheUserAboutError("No label printing because of communication error (Shipment).", session);
			}

			return shipmentId;
		}

		public string getLabel(PmxDbConnection conn, Session session, string geoSession, int shipmentId)
		{

			string printString = null;
			ServicePointManager.Expect100Continue = true;
			ServicePointManager.SecurityProtocol = (SecurityProtocolType)3072;
			ServicePointManager.DefaultConnectionLimit = 9999;

		
			var httpWebRequest = (HttpWebRequest)WebRequest.Create(String.Format(Definitions.URL_LABEL, shipmentId));
			httpWebRequest.ContentType = "application/json";
			httpWebRequest.Method = "GET";
			httpWebRequest.Headers.Add("GeoSession", geoSession);			
			httpWebRequest.Accept = "text/html";

		
			RestApiResponse<WebServiceShipmentResponse> response = new RestApiResponse<WebServiceShipmentResponse>();
			var httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();

			using (var streamReader = new StreamReader(httpWebResponse.GetResponseStream()))
			{
				response.ResponseString = streamReader.ReadToEnd();
			}			

			if (response != null)
			{
				if (string.IsNullOrEmpty(response.ErrorMessage) && response.ResponseHttpStatus == 0)
				{
					printString = response.ResponseString;					
				}
				else
				{
					LogError(response.ErrorMessage);
					InformTheUserAboutError(response.ErrorMessage, session);					
				}
			}

			else
			{
				LogError("The response wasn't a valid json string (label).");
				InformTheUserAboutError("No label printing because of communication error (label).", session);
				
			}

			return printString;
		}

		public string CreateJsonContent(PmxDbConnection conn, int docEntry)
		{
			
			PmxItemTransactionalInfoProvider itriProv = new PmxItemTransactionalInfoProvider(conn);

			StringBuilder strJson = new StringBuilder();

			DataSet ds = new DataSet();
			DataTable tbl = new DataTable();
			DataTable tblInt = new DataTable();


			string query = BuildQuery.GetQueryGetDataFromView(docEntry, Definitions.VIEW_NAME_DPD, Definitions.VIEW_COLUMN_DOCENTRY, conn.DbTool);
		
			using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
			{
				if (rs.EoF)
					return null;
				 ds = SboRecordsetHelper.ConvertRecordsetToDataSet(s_log, rs);
				tbl = ds.Tables[0];
			}

			if (tbl.Rows.Count > 0)
			{			
				
				strJson.Append("{");
			
				var arrayNames = (from DataColumn x in tbl.Columns
								  where !x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_COSIGNMENT)
								  && !x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_COLLECTION_DETAILS)
								  && !x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_DELIVERY_DETAILS)
								  && !x.ColumnName.Equals(Definitions.DPD_XMLNODE_DOCENTRY)
								  select x.ColumnName).ToArray();

			
				foreach (var item in arrayNames)
				{
					strJson.Append(ConvertTo(item, item, tbl.Rows[0][item].ToString()));
				}
				

				//Consignment
				strJson.AppendFormat("\"{0}\": [", Definitions.DPD_XMLNODE_COSIGNMENT);
				strJson.Append("{");

				arrayNames = (from DataColumn x in tbl.Columns
							  where x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_COSIGNMENT)
							  select x.ColumnName).ToArray();

				foreach (var item in arrayNames)
				{
					strJson.Append(ConvertTo(item, item.Replace(Definitions.VIEW_PREFIX_COSIGNMENT, string.Empty), tbl.Rows[0][item].ToString()));
				}
				//strJson.Remove(strJson.Length - 1, 1);

				//CollectionDetails
				strJson.AppendFormat("\"{0}\":", Definitions.DPD_XMLNODE_COLLECTION_DETAILS);
				strJson.Append("{");

				//contactDetails
				strJson.AppendFormat("\"{0}\":", Definitions.DPD_XMLNODE_CONTACTDETAILS);
				strJson.Append("{");
				arrayNames = (from DataColumn x in tbl.Columns
							  where x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_COLLECTION_DETAILS + Definitions.VIEW_PREFIX_CONTACTDETAILS)
							  select x.ColumnName).ToArray();

				foreach (var item in arrayNames)
				{
					strJson.Append(ConvertTo(item, item.Replace(Definitions.VIEW_PREFIX_COLLECTION_DETAILS + Definitions.VIEW_PREFIX_CONTACTDETAILS, string.Empty), tbl.Rows[0][item].ToString()));
				}
				strJson.Remove(strJson.Length - 1, 1);
				strJson.Append("},");//End contactDetails

				//address
				strJson.AppendFormat("\"{0}\":", Definitions.DPD_XMLNODE_ADDRESS);
				strJson.Append("{");

				arrayNames = (from DataColumn x in tbl.Columns
							  where x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_COLLECTION_DETAILS + Definitions.VIEW_PREFIX_ADDRESS)
							  select x.ColumnName).ToArray();

				foreach (var item in arrayNames)
				{
					strJson.Append(ConvertTo(item, item.Replace(Definitions.VIEW_PREFIX_COLLECTION_DETAILS + Definitions.VIEW_PREFIX_ADDRESS, string.Empty), tbl.Rows[0][item].ToString()));
				}
				strJson.Remove(strJson.Length - 1, 1);

				strJson.Append("}"); //End address
				strJson.Append("},"); //End CollectionDetails

				//DeliveryDetails
				strJson.AppendFormat("\"{0}\":", Definitions.DPD_XMLNODE_DELIVERY_DETAILS);
				strJson.Append("{");


				//contactDetails
				strJson.AppendFormat("\"{0}\":", Definitions.DPD_XMLNODE_CONTACTDETAILS);
				strJson.Append("{");

				arrayNames = (from DataColumn x in tbl.Columns
							  where x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_DELIVERY_DETAILS + Definitions.VIEW_PREFIX_CONTACTDETAILS)
							  select x.ColumnName).ToArray();

				foreach (var item in arrayNames)
				{
					strJson.Append(ConvertTo(item, item.Replace(Definitions.VIEW_PREFIX_DELIVERY_DETAILS + Definitions.VIEW_PREFIX_CONTACTDETAILS, string.Empty), tbl.Rows[0][item].ToString()));
				}
				strJson.Remove(strJson.Length - 1, 1);
				strJson.Append("},");//End contactDetails

				//address
				strJson.AppendFormat("\"{0}\":", Definitions.DPD_XMLNODE_ADDRESS);
				strJson.Append("{");



				arrayNames = (from DataColumn x in tbl.Columns
							  where x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_DELIVERY_DETAILS + Definitions.VIEW_PREFIX_ADDRESS)
							  select x.ColumnName).ToArray();

				foreach (var item in arrayNames)
				{
					strJson.Append(ConvertTo(item, item.Replace(Definitions.VIEW_PREFIX_DELIVERY_DETAILS + Definitions.VIEW_PREFIX_ADDRESS, string.Empty), tbl.Rows[0][item].ToString()));
				}
				strJson.Remove(strJson.Length - 1, 1);
				strJson.Append("},"); //End address

				//notificationDetails
				strJson.AppendFormat("\"{0}\":", Definitions.DPD_XMLNODE_NOTIFICATIONDETAILS);
				strJson.Append("{");

				arrayNames = (from DataColumn x in tbl.Columns
							  where x.ColumnName.StartsWith(Definitions.VIEW_PREFIX_DELIVERY_DETAILS + Definitions.VIEW_PREFIX_NOTIFICATIONDETAILS)
							  select x.ColumnName).ToArray();

				foreach (var item in arrayNames)
				{
					strJson.Append(ConvertTo(item, item.Replace(Definitions.VIEW_PREFIX_DELIVERY_DETAILS + Definitions.VIEW_PREFIX_NOTIFICATIONDETAILS, string.Empty), tbl.Rows[0][item].ToString()));
				}
				strJson.Remove(strJson.Length - 1, 1);
				strJson.Append("}"); //End notificationDetails
				strJson.Append("}"); //End DeliveryDetails

				strJson.Append("}]");//End Consignment
				strJson.Append("}"); // End
			}
			else
			{
				//No export needed
				return null;
			}
			s_log.Debug("JSon:" + strJson.ToString());			
			return strJson.ToString();
		}

		

		#endregion

		#region HELPERS

		private void LogError(string message)
		{
			s_log.Error(message);
		}

		private void InformTheUserAboutError(string message, Session session)
		{
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"MessageKey", "Integration: " + message,
					"NoTranslationOfMessageKey", true,
					"IsError", true,
					"ShowButton", true));
			WaitForMessage();
		}

		private string ConvertTo(string columnName, string name, object value)
		{
			string returnValue = "";
			if (value.ToString() == "") {
				returnValue = String.Format("\"{0}\": null", name);
			}
			else
			{
				returnValue = String.Format("\"{0}\":\"{1}\"", name, value.ToString());
			}

			if (ListColumnsDecimals.Contains(columnName))
			{
				decimal dec = String.IsNullOrEmpty(value.ToString()) ? 0 : Convert.ToDecimal(value);
				returnValue= String.Format("\"{0}\":{1}", name, dec.ToString(Definitions.DPD_DECIMAL_FORMAT, CultureInfo.InvariantCulture));				
			}

			if (ListColumnsDates.Contains(columnName))
			{
				DateTime date = Convert.ToDateTime(value);
				returnValue = String.Format("\"{0}\":\"{1}\"", name, date.ToString(Definitions.DPD_DATE_FORMAT, CultureInfo.InvariantCulture));
			
			}
			if (ListColumnsInteger.Contains(columnName))
			{
				int integer = Convert.ToInt32(value);
				returnValue = String.Format("\"{0}\":{1}", name, integer.ToString());			
			}

			if (ListColumnsBoolean.Contains(columnName))
			{
				returnValue = String.Format("\"{0}\":{1}", name, value.ToString());
			}
			
			return string.Format ("{0},", returnValue);
		}

		private bool PrintDoc()
		{
			try
			{
				System.Drawing.Printing.PrintDocument doc = new System.Drawing.Printing.PrintDocument();
				doc.DocumentName = "DPD";
				doc.DefaultPageSettings.Landscape = false;
				doc.PrinterSettings.PrinterName =  m_printer;
				doc.PrintController = new System.Drawing.Printing.StandardPrintController();
				doc.PrinterSettings.PrintFileName = fileName;
				doc.Print();

			
			}
			catch (Exception ex)
			{
				throw ex;
			}

			return true;

		}
		#endregion

		#region BUILD QUERY

		internal static class BuildQuery
		{
			/// <summary>
			/// Gets the query get data from view .
			/// </summary>
			/// <param name="docEntry">The docentry.</param>
			/// <param name="viewName">The view name.</param>
			/// <param name="viewName">The column name for docEntry.</param>
			/// <param name="dbTool">The db tool.</param>
			/// <returns></returns>
			public static string GetQueryGetDataFromView(int docEntry, string viewName, string columnNameKey, DbTool dbTool)
			{
				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

				strBuilder.
					Select.Append(" * ").
					From.
						Append(viewName).CRLF.
					Where.
						EscColumnName(columnNameKey).Eq.
						Append(SqlCommandHelper.FormatColumnValue(docEntry, dbTool.CultureInfo));

				return strBuilder.ToString();



			}
		}
		#endregion
	}


	/// <summary>
	/// The webservice response.
	/// </summary>
	public class WebServiceLoginResponse
	{		
		public string error { get; set; }
		public WebServiceLoginResponseData data { get; set; }
	}
	/// <summary>
	/// The webservice response.
	/// </summary>	
	public class WebServiceLoginResponseData
	{
		public string geoSession { get; set; }
		public string flag { get; set; }
	}

	public class WebServiceShipmentResponse
	{
		public List<WebServiceResponseError> error { get; set; }
		public WebServiceShipmentResponseData data { get; set; }
	}

	/// <summary>
	/// The webservice response.
	/// </summary>	
	public class WebServiceShipmentResponseData
	{
		public int shipmentId { get; set; }
	}
	
	/// <summary>
	/// The webservice response.
	/// </summary>
	public class WebServiceRequest
	{
	}
	/// <summary>
	/// The webservice response.
	/// </summary>
	public class WebServiceResponseError
	{
		public string errorCode { get; set; }
		public string obj { get; set; }
		public string errorType { get; set; }
		public string errorMessage { get; set; }
		public string errorAction { get; set; }
	}

}
